local FOLDER_NAME, private = ...
local core = LibStub("AceAddon-3.0"):GetAddon(FOLDER_NAME)
local L = LibStub("AceLocale-3.0"):GetLocale(FOLDER_NAME)
local ACR = LibStub("AceConfigRegistry-3.0")
local version = GetAddOnMetadata(FOLDER_NAME, "X-Version")
local author = GetAddOnMetadata(FOLDER_NAME, "X-Author")
local builddate = GetAddOnMetadata(FOLDER_NAME, "X-Build")
local playerName = UnitName("player")
local pairs, type = pairs, type
local getOpt, setOpt

do
    function getOpt(info)
        local key = info[#info]
        return core.db.profile[key]
    end

    function setOpt(info, value)
        local key = info[#info]
        core.db.profile[key] = value
        core:applySettings()
    end
end

local BGMDropDownMenu = CreateFrame("Frame", "BGMDropDownMenu")
BGMDropDownMenu.displayMode = "MENU"
BGMDropDownMenu.info = {}
BGMDropDownMenu.uncheck = function(dropdownbutton)
    _G[dropdownbutton:GetName().."Check"]:Hide()
end

local info = {}
BGMDropDownMenu.initialize = function(self, level)
    if not level then return end
    local info = self.info
    wipe(info)
    if level == 1 then
        info.keepShownOnClick = 1
        info.isTitle = 1
        info.text = "Broker Gems"
        info.notCheckable = 1
        UIDropDownMenu_AddButton(info, level)
        info.isTitle = nil
        info.text = ""
        info.disabled = 1
        UIDropDownMenu_AddButton(info, level)
        info.isTitle = 1
        info.text = L["Tracking Options"]
        UIDropDownMenu_AddButton(info, level)
        info.isTitle = nil
        info.disabled = nil
        info.hasArrow = 1
        info.text = ("|cffffff99"..L["Vanilla"].."|r")
        info.value = "submenu1"
        info.func = self.uncheck
        UIDropDownMenu_AddButton(info, level)
        info.text = ("|cff00ff00"..L["The Burning Crusade"].."|r")
        info.value = "submenu2"
        info.func = self.uncheck
        UIDropDownMenu_AddButton(info, level)
        info.text = ("|cff0066ff"..L["Wrath of the Lich King"].."|r")
        info.value = "submenu3"
        info.func = self.uncheck
        UIDropDownMenu_AddButton(info, level)
        info.text = ("|cffff6600"..L["Cataclysm"].."|r")
        info.value = "submenu4"
        info.func = self.uncheck
        UIDropDownMenu_AddButton(info, level)
        info.text = ("|cff19ac87"..L["Mists of Pandaria"].."|r")
        info.value = "submenu5"
        info.func = self.uncheck
        UIDropDownMenu_AddButton(info, level)
        info.hasArrow = nil
        info.notCheckable = nil
        info.text = L["Show Icons"]
        info.func = function() core.db.profile.showIcons = not core.db.profile.showIcons core:applySettings() end
        info.checked = function() return core.db.profile.showIcons end
        UIDropDownMenu_AddButton(info, level)
        info.text = L["Show Short Text"]
        info.func = function() core.db.profile.showShort = not core.db.profile.showShort core:applySettings() end
        info.checked = function() return core.db.profile.showShort end
        UIDropDownMenu_AddButton(info, level)
        info.text = L["Show Icons Only"]
        info.func = function() core.db.profile.iconsOnly = not core.db.profile.iconsOnly core:applySettings() end
        info.checked = function() return core.db.profile.iconsOnly end
        UIDropDownMenu_AddButton(info, level)
        info.notCheckable = 1
        info.text = ""
        info.disabled = 1
        UIDropDownMenu_AddButton(info, level)
        info.isTitle = 1
        info.text = L["Tooltip Options"]
        UIDropDownMenu_AddButton(info, level)
        info.isTitle = nil
        info.notCheckable = nil
        info.disabled = nil
        info.text = ("|cffffff99"..L["Vanilla"].."|r")
        info.func = function() core.db.profile.modules.gmv = not core.db.profile.modules.gmv core:applySettings() end
        info.checked = function() return core.db.profile.modules.gmv end
        UIDropDownMenu_AddButton(info, level)
        info.text = ("|cff00ff00"..L["The Burning Crusade"].."|r")
        info.func = function() core.db.profile.modules.gmbc = not core.db.profile.modules.gmbc core:applySettings() end
        info.checked = function() return core.db.profile.modules.gmbc end
        UIDropDownMenu_AddButton(info, level)
        info.text = ("|cff0066ff"..L["Wrath of the Lich King"].."|r")
        info.func = function() core.db.profile.modules.gmlk = not core.db.profile.modules.gmlk core:applySettings() end
        info.checked = function() return core.db.profile.modules.gmlk end
        UIDropDownMenu_AddButton(info, level)
        info.text = ("|cffff6600"..L["Cataclysm"].."|r")
        info.func = function() core.db.profile.modules.gmcata = not core.db.profile.modules.gmcata core:applySettings() end
        info.checked = function() return core.db.profile.modules.gmcata end
        UIDropDownMenu_AddButton(info, level)
        info.text = ("|cff19ac87"..L["Mists of Pandaria"].."|r")
        info.func = function() core.db.profile.modules.gmmop = not core.db.profile.modules.gmmop core:applySettings() end
        info.checked = function() return core.db.profile.modules.gmmop end
        UIDropDownMenu_AddButton(info, level)
        info.text = L["Include Zero Counts"]
        info.func = function() core.db.profile.showZero = not core.db.profile.showZero core:applySettings() end
        info.checked = function() return core.db.profile.showZero end
        UIDropDownMenu_AddButton(info, level)
        info.text = L["Include Bank Counts"]
        info.func = function() core.db.profile.showBank = not core.db.profile.showBank core:applySettings() end
        info.checked = function() return core.db.profile.showBank end
        UIDropDownMenu_AddButton(info, level)
        info.text = L["Include Alt Counts"]
        info.func = function() core.db.profile.showAlts = not core.db.profile.showAlts core:applySettings() end
        info.checked = function() return core.db.profile.showAlts end
        UIDropDownMenu_AddButton(info, level)
        info.text = L["Include Guild Counts"]
        info.func = function() core.db.profile.showGuild = not core.db.profile.showGuild core:applySettings() end
        info.checked = function() return core.db.profile.showGuild end
        UIDropDownMenu_AddButton(info, level)
        info.text = L["Show Icons"]
        info.func = function() core.db.profile.tooltipIcons = not core.db.profile.tooltipIcons core:applySettings() end
        info.checked = function() return core.db.profile.tooltipIcons end
        UIDropDownMenu_AddButton(info, level)
        info.text = L["Show Short Text"]
        info.func = function() core.db.profile.tooltipShowShort = not core.db.profile.tooltipShowShort core:applySettings() end
        info.checked = function() return core.db.profile.tooltipShowShort end
        UIDropDownMenu_AddButton(info, level)
        info.text = L["Show Icons Only"]
        info.func = function() core.db.profile.tooltipIconsOnly = not core.db.profile.tooltipIconsOnly core:applySettings() end
        info.checked = function() return core.db.profile.tooltipIconsOnly end
        UIDropDownMenu_AddButton(info, level)
        info.text = L["Color lines in the tooltip."]
        info.func = function() core.db.profile.lineColor = not core.db.profile.lineColor core:applySettings() end
        info.checked = function() return core.db.profile.lineColor end
        UIDropDownMenu_AddButton(info, level)
        info.text = L["Color text in the tooltip."]
        info.func = function() core.db.profile.textColor = not core.db.profile.textColor core:applySettings() end
        info.checked = function() return core.db.profile.textColor end
        UIDropDownMenu_AddButton(info, level)
        info.notCheckable = 1
        info.text = ""
        info.disabled = 1
        UIDropDownMenu_AddButton(info, level)
        info.disabled = nil
        info.text = L["Close"]
        info.func = function() CloseDropDownMenus() end
        UIDropDownMenu_AddButton(info, level)
    elseif level == 2 then
        if UIDROPDOWNMENU_MENU_VALUE == "submenu1" then
            info.keepShownOnClick = 1
            for k,itemid in pairs (private.vangems.id) do
                local color = private.vangems.color[k]
                if not core.db.profile.textColor then color = "FFFFFF" end
                local name = private.vangems.strings[k]
                local fixeditem = strsub(itemid, 1, -1)
                local item  = "|cff"..color..name.."|r"
                info.text = item
                info.func = function() core.db.profile.tracking[fixeditem] = not core.db.profile.tracking[fixeditem] core:applySettings() end
                info.checked = function() return core.db.profile.tracking[fixeditem] end
                UIDropDownMenu_AddButton(info, level)
            end
        end
        if UIDROPDOWNMENU_MENU_VALUE == "submenu2" then
            info.keepShownOnClick = 1
            for k,itemid in pairs (private.bcgems.id) do
                local color = private.bcgems.color[k]
                if not core.db.profile.textColor then color = "FFFFFF" end
                local name = private.bcgems.strings[k]
                local fixeditem = strsub(itemid, 1, -1)
                local item  = "|cff"..color..name.."|r"
                info.text = item
                info.func = function() core.db.profile.tracking[fixeditem] = not core.db.profile.tracking[fixeditem] core:applySettings() end
                info.checked = function() return core.db.profile.tracking[fixeditem] end
                UIDropDownMenu_AddButton(info, level)
            end
        end
        if UIDROPDOWNMENU_MENU_VALUE == "submenu3" then
            info.keepShownOnClick = 1
            for k,itemid in pairs (private.lkgems.id) do
                local color = private.lkgems.color[k]
                if not core.db.profile.textColor then color = "FFFFFF" end
                local name = private.lkgems.strings[k]
                local fixeditem = strsub(itemid, 1, -1)
                local item = "|cff"..color..name.."|r"
                info.text = item
                info.func = function() core.db.profile.tracking[fixeditem] = not core.db.profile.tracking[fixeditem] core:applySettings() end
                info.checked = function() return core.db.profile.tracking[fixeditem] end
                UIDropDownMenu_AddButton(info, level)
            end
        end
        if UIDROPDOWNMENU_MENU_VALUE == "submenu4" then
            info.keepShownOnClick = 1
            for k,itemid in pairs (private.catagems.id) do
                local color = private.catagems.color[k]
                if not core.db.profile.textColor then color = "FFFFFF" end
                local name = private.catagems.strings[k]
                local fixeditem = strsub(itemid, 1, -1)
                local item = "|cff"..color..name.."|r"
                info.text = item
                info.func = function() core.db.profile.tracking[fixeditem] = not core.db.profile.tracking[fixeditem] core:applySettings() end
                info.checked = function() return core.db.profile.tracking[fixeditem] end
                UIDropDownMenu_AddButton(info, level)
            end
        end
        if UIDROPDOWNMENU_MENU_VALUE == "submenu5" then
            info.keepShownOnClick = 1
            for k,itemid in pairs (private.mopgems.id) do
                local color = private.mopgems.color[k]
                if not core.db.profile.textColor then color = "FFFFFF" end
                local name = private.mopgems.strings[k]
                local fixeditem = strsub(itemid, 1, -1)
                local item = "|cff"..color..name.."|r"
                info.text = item
                info.func = function() core.db.profile.tracking[fixeditem] = not core.db.profile.tracking[fixeditem] core:applySettings() end
                info.checked = function() return core.db.profile.tracking[fixeditem] end
                UIDropDownMenu_AddButton(info, level)
            end
        end
    end
end

local options, moduleOptions = nil, {}
local function getOptions()
    if not options then
         options = {
            type = "group",
            name = "Broker Gems",
            args = {
                display = {
                    order = 5,
                    type = "group",
                    inline = true,
                    name = "",
                    args = {
                        author = {
                            order = 10,
                            type = "description",
                            name = L["Author: "] .. author .. "    "..L["Version: "] .. version .. "    "..L["Build Date: "] .. builddate
                        },
                        counts = {
                            type = "group",
                            name = L["Display Options"],
                            guiInline = true,
                            order = 15,
                            get = getOpt,
                            set = setOpt,
                            args = {
                                showZero = {
                                    type = "toggle",
                                    order = 20,
                                    width = "double",
                                    name = L["Include Zero Counts"],
                                    get = getOpt,
                                    set = setOpt,
                                },
                                showBank = {
                                    type = "toggle",
                                    order = 25,
                                    width = "double",
                                    name = L["Include Bank Counts"],
                                    get = getOpt,
                                    set = setOpt,
                                },
                                showAlts = {
                                    type = "toggle",
                                    order = 30,
                                    width = "double",
                                    name = L["Include Alt Counts"],
                                    get = getOpt,
                                    set = setOpt,
                                },
                                showGuild = {
                                    type = "toggle",
                                    order = 35,
                                    width = "double",
                                    name = L["Include Guild Counts"],
                                    get = getOpt,
                                    set = setOpt,
                                },
                            },
                        },
                        tooltip = {
                            type = "group",
                            name = L["Tooltip Options"],
                            guiInline = true,
                            order = 40,
                            get = getOpt,
                            set = setOpt,
                            args = {
                                scale = {
                                    order = 45,
                                    type = "range",
                                    name = L["Tooltip Scale"],
                                    min = .5,
                                    max = 1.5,
                                    step = .05,
                                    get = getOpt,
                                    set = setOpt,
                                },
                                scroll= {
                                    order = 50,
                                    type = "range",
                                    name = L["Tooltip Scroll Height"],
                                    min = 100,
                                    max = 900,
                                    step = 25,
                                    get = getOpt,
                                    set = setOpt,
                                },
                                updatetime = {
                                    order = 55,
                                    type = "range",
                                    name = L["Update Time"],
                                    min = 1,
                                    max = 30,
                                    step = 1,
                                    get = getOpt,
                                    set = setOpt,
                                },
                                scantime = {
                                    order = 60,
                                    type = "range",
                                    name = L["Guild Bank Scan Time"],
                                    min = 5,
                                    max = 30,
                                    step = 1,
                                    get = getOpt,
                                    set = setOpt,
                                },
                                tooltipIcons = {
                                    type = "toggle",
                                    order = 65,
                                    width = "double",
                                    name = L["Show icons in the tooltip."],
                                    get = getOpt,
                                    set = setOpt,
                                },
                                tooltipShowShort = {
                                    type = "toggle",
                                    order = 70,
                                    width = "double",
                                    name = L["Show Short Text"],
                                    get = getOpt,
                                    set = setOpt,
                                },
                                tooltipIconsOnly = {
                                    type = "toggle",
                                    order = 75,
                                    width = "double",
                                    name = L["Show icons only in the tooltip."],
                                    get = getOpt,
                                    set = setOpt,
                                    hidden = function() return not core.db.profile.tooltipIcons end
                                },
                                lineColor = {
                                    type = "toggle",
                                    order = 80,
                                    width = "double",
                                    name = L["Color lines in the tooltip."],
                                    get = getOpt,
                                    set = setOpt,
                                },
                                textColor = {
                                    type = "toggle",
                                    order = 85,
                                    width = "double",
                                    name = L["Color text in the tooltip."],
                                    get = getOpt,
                                    set = setOpt,
                                },
                            },
                        },
                        tracking = {
                            type = "group",
                            name = L["Tracking Options"],
                            guiInline = true,
                            order = 85,
                            get = getOpt,
                            set = setOpt,
                            args = {
                                showIcons= {
                                    type = "toggle",
                                    order = 90,
                                    width = "double",
                                    name = L["Show Icons"],
                                    get = getOpt,
                                    set = setOpt,
                                },
                                showShort = {
                                    type  = "toggle",
                                    order = 95,
                                    width = "double",
                                    name = L["Show Short Text"],
                                    get = getOpt,
                                    set = setOpt,
                                },
                                iconsOnly = {
                                    type = "toggle",
                                    order = 100,
                                    width = "double",
                                    name = L["Show Icons Only"],
                                    get = getOpt,
                                    set = setOpt,
                                    hidden = function() return not core.db.profile.showIcons end
                                },
                            },
                        },
                        purge = {
                            type = "group",
                            name = L["Delete Character"],
                            guiInline = true,
                            order = 105,
                            args = {
                                deleteChar= {
                                    type = "select",
                                    order = 110,
                                    name = L["Delete Character"],
                                    desc = L["This action will force a UI reload."],
                                    values = private.charsList,
                                    set = function(v, alt)
                                            core:purgeData(alt)
                                    end
                                },
                            },
                        },
                    },
                },
            },
        }
        for k,v in pairs(moduleOptions) do
            options.args[k] = (type(v) == "function") and v() or v
        end
    end
    return options
end

function core:SetupOptions()
    self.optFrames = {}
    ACR:RegisterOptionsTable(FOLDER_NAME, getOptions)
    self.optFrames.core = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(FOLDER_NAME, "Broker Gems", nil, "display")
    self.optFrames.core.default = function() core:SetDefaultOptions() end
    self:RegisterModuleOptions("Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(core.db), "Profiles")
end

function core:SetDefaultOptions()
    core.db:ResetProfile()
    ACR:NotifyChange(FOLDER_NAME)
end

function core:RegisterModuleOptions(name, optTable, displayName)
    moduleOptions[name] = optTable
    self.optFrames[name] = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(FOLDER_NAME, displayName or name, "Broker Gems", name)
end

function core:purgeData(alt)
   core.db.realm.chars[alt] = nil
   ReloadUI()
end