local FOLDER_NAME, private = ...
local core = LibStub("AceAddon-3.0"):GetAddon(FOLDER_NAME)
local L = LibStub("AceLocale-3.0"):GetLocale(FOLDER_NAME)
local strformat, pairs, tinsert, unpack, strsub  = string.format, pairs, tinsert, unpack, string.sub
local GetItemIcon = GetItemIcon
local LTT = LibStub:GetLibrary("LibQTip-1.0")
local tooltip, xpr, xpg, xpb, xpa

local classColors = {}
for class, color in pairs(CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS) do
    classColors[class] = strformat( "%2x%2x%2x", color.r * 255, color.g * 255, color.b * 255 )
end

if CUSTOM_CLASS_COLORS then
    CUSTOM_CLASS_COLORS:RegisterCallback(function()
        for class, color in pairs(CUSTOM_CLASS_COLORS) do
            classColors[class] = strformat("%2x%2x%2x", color.r * 255, color.g * 255, color.b * 255)
        end
        end)
end

local bgm = LibStub("LibDataBroker-1.1"):NewDataObject("Broker Gems",{
    type  = "data source",
    text  = " ",
    label = "Broker Gems",
    icon  = "Interface\\AddOns\\"..FOLDER_NAME.."\\icon",
    OnClick = function(self, button)
        if LTT:IsAcquired("BGMToolTip") then
            tooltip:Hide()
            tooltip:Clear()
        end
        if button == "RightButton" then
            if InterfaceOptionsFrame:IsShown() then
                HideUIPanel(InterfaceOptionsFrame)
            end
            ToggleDropDownMenu(1, nil, BGMDropDownMenu, "cursor")
        elseif
            button == "LeftButton" then
            InterfaceOptionsFrame_OpenToCategory(core.optFrames.core)
        end
    end,
    OnEnter = function(self)
        if LTT:IsAcquired("BGMToolTip") then
            tooltip:Hide()
            tooltip:Clear()
        else
            tooltip = LTT:Acquire("BGMToolTip", 2, "LEFT", "LEFT")
        end
        tooltip:SetScale(core.db.profile.scale)
        core:TooltipDraw()
        tooltip:SmartAnchorTo(self)
        tooltip:SetAutoHideDelay( .1, self )
        tooltip:EnableMouse()
        tooltip:SetScrollStep(core.db.profile.scroll - 25)
        tooltip:UpdateScrolling(core.db.profile.scroll)
        tooltip:Show()
    end,
    OnLeave = function(self)
    end,
} )

private.bgm = bgm

function core:onTextUpdate()
    if not private.scanPending then
        local tracktext = ""
        local textcount = 0
        for k,itemid in pairs (private.vangems.id) do
            local iconpath = GetItemIcon(itemid)
            local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
            local color = private.vangems.color[k]
            if not core.db.profile.textColor then color = "FFFFFF" end
            local name = private.vangems.strings[k]
            local shortname = private.vangems.short[k]
            local text = core:createText(itemid, iconpath, icon, color, name, shortname, item)
            if text ~= nil then
                textcount = textcount + 1
                if textcount == 1 then
                    tracktext = tracktext.." "..(text).." "..core:Color(core.db.realm.chars[self.charName].data[itemid].inv)
                else
                    tracktext = tracktext.."  "..(text).." "..core:Color(core.db.realm.chars[self.charName].data[itemid].inv)
                end
            end
        end
        for k,itemid in pairs (private.bcgems.id) do
            local iconpath = GetItemIcon(itemid)
            local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
            local color = private.bcgems.color[k]
            if not core.db.profile.textColor then color = "FFFFFF" end
            local name = private.bcgems.strings[k]
            local shortname = private.bcgems.short[k]
            local text = core:createText(itemid, iconpath, icon, color, name, shortname, item)
            if text ~= nil then
                textcount = textcount + 1
                if textcount == 1 then
                    tracktext = tracktext.." "..(text).." "..core:Color(core.db.realm.chars[self.charName].data[itemid].inv)
                else
                    tracktext = tracktext.."  "..(text).." "..core:Color(core.db.realm.chars[self.charName].data[itemid].inv)
                end
            end
        end
        for k,itemid in pairs (private.lkgems.id) do
            local iconpath = GetItemIcon(itemid)
            local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
            local color = private.lkgems.color[k]
            if not core.db.profile.textColor then color = "FFFFFF" end
            local name = private.lkgems.strings[k]
            local shortname = private.lkgems.short[k]
            local text = core:createText(itemid, iconpath, icon, color, name, shortname, item)
            if text ~= nil then
                textcount = textcount + 1
                if textcount == 1 then
                    tracktext = tracktext.." "..(text).." "..core:Color(core.db.realm.chars[self.charName].data[itemid].inv)
                else
                    tracktext = tracktext.."  "..(text).." "..core:Color(core.db.realm.chars[self.charName].data[itemid].inv)
                end
            end
        end
        for k,itemid in pairs (private.catagems.id) do
            local iconpath = GetItemIcon(itemid)
            local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
            local color = private.catagems.color[k]
            if not core.db.profile.textColor then color = "FFFFFF" end
            local name = private.catagems.strings[k]
            local shortname = private.catagems.short[k]
            local text = core:createText(itemid, iconpath, icon, color, name, shortname, item)
            if text ~= nil then
                textcount = textcount + 1
                if textcount == 1 then
                    tracktext = tracktext.." "..(text).." "..core:Color(core.db.realm.chars[self.charName].data[itemid].inv)
                else
                    tracktext = tracktext.."  "..(text).." "..core:Color(core.db.realm.chars[self.charName].data[itemid].inv)
                end
            end
        end
        for k,itemid in pairs (private.mopgems.id) do
            local iconpath = GetItemIcon(itemid)
            local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
            local color = private.mopgems.color[k]
            if not core.db.profile.textColor then color = "FFFFFF" end
            local name = private.mopgems.strings[k]
            local shortname = private.mopgems.short[k]
            local text = core:createText(itemid, iconpath, icon, color, name, shortname, item)
            if text ~= nil then
                textcount = textcount + 1
                if textcount == 1 then
                    tracktext = tracktext.." "..(text).." "..core:Color(core.db.realm.chars[self.charName].data[itemid].inv)
                else
                    tracktext = tracktext.."  "..(text).." "..core:Color(core.db.realm.chars[self.charName].data[itemid].inv)
                end
            end
        end
        if textcount == 0 then
            tracktext = tracktext.." "..("|cff00ff00"..L["Not Tracking"].."|r")
        end
       bgm.text = tracktext
    end
end

function core:createText(itemid, iconpath, icon, color, name, shortname, item)
    local item
    local fixeditem = strsub(itemid, 1, -1)
    if core.db.profile.tracking[fixeditem] then
        if core.db.profile.tracking[fixeditem] then
            if not core.db.profile.showShort then
                item  = "|cff"..color..name.."|r"
            else
                item = "|cff"..color..shortname.."|r"
            end
            if core.db.profile.showIcons and not core.db.profile.iconsOnly then
                item = icon..item
            elseif core.db.profile.showIcons and core.db.profile.iconsOnly then
                item = icon
            else
                item = item
            end
        end
    end
    return item
end

function core:TooltipDraw()
    local y ,x, justification = tooltip:AddHeader(), 2, {}
    local name = "|cff"..classColors[core.db.realm.chars[self.charName].class]..strsub(self.charName, 1, 10).."|r"
    tooltip:SetCell(y, 2, "  "..name)
    tooltip:AddSeparator(2,1,1,1)
    if core.db.profile.showAlts then
        for char, _ in pairs(core.db.realm.chars) do
            tinsert(justification, "LEFT")
            if char ~= self.charName then
                x = x + 1
                tooltip:SetColumnLayout(x, "LEFT", "LEFT", unpack(justification))
                local name = "|cff"..classColors[core.db.realm.chars[char].class]..strsub(char, 1, 10).."|r"
                tooltip:SetCell(y, x, "  "..name)
            end
        end
    end
    if core.db.profile.modules.gmv then
    xpr, xpg, xpb, xpa = 1, 1, 0, .1
        for k, itemid in pairs(private.vangems.id) do
            local iconpath = GetItemIcon(itemid)
            local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
            local color = private.vangems.color[k]
            if not core.db.profile.textColor then color = "FFFFFF" end
            local name = private.vangems.strings[k]
            local shortname = private.vangems.short[k]
            core:createTooltip(itemid, iconpath, icon, color, name, shortname, xpr, xpg, xpb, xpa)
        end
    end
    if core.db.profile.modules.gmbc then
    xpr, xpg, xpb, xpa = 0, 1, 0, .1
        for k, itemid in pairs(private.bcgems.id) do
            local iconpath = GetItemIcon(itemid)
            local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
            local color = private.bcgems.color[k]
            if not core.db.profile.textColor then color = "FFFFFF" end
            local name = private.bcgems.strings[k]
            local shortname = private.bcgems.short[k]
            core:createTooltip(itemid, iconpath, icon, color, name, shortname, xpr, xpg, xpb, xpa)
        end
    end
    if core.db.profile.modules.gmlk then
    xpr, xpg, xpb, xpa = 0, .1 ,1, .1
        for k, itemid in pairs(private.lkgems.id) do
            local iconpath = GetItemIcon(itemid)
            local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
            local color = private.lkgems.color[k]
            if not core.db.profile.textColor then color = "FFFFFF" end
            local name = private.lkgems.strings[k]
            local shortname = private.lkgems.short[k]
            core:createTooltip(itemid, iconpath, icon, color, name, shortname, xpr, xpg, xpb, xpa)
        end
    end
    if core.db.profile.modules.gmcata then
    xpr, xpg, xpb, xpa = .9, .2, 0, .1
        for k, itemid in pairs(private.catagems.id) do
            local iconpath = GetItemIcon(itemid)
            local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
            local color = private.catagems.color[k]
            if not core.db.profile.textColor then color = "FFFFFF" end
            local name = private.catagems.strings[k]
            local shortname = private.catagems.short[k]
            core:createTooltip(itemid, iconpath, icon, color, name, shortname, xpr, xpg, xpb, xpa)
        end
    end
    if core.db.profile.modules.gmmop then
    xpr, xpg, xpb, xpa = .2, .8, .9, .1
        for k, itemid in pairs(private.mopgems.id) do
            local iconpath = GetItemIcon(itemid)
            local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
            local color = private.mopgems.color[k]
            if not core.db.profile.textColor then color = "FFFFFF" end
            local name = private.mopgems.strings[k]
            local shortname = private.mopgems.short[k]
            core:createTooltip(itemid, iconpath, icon, color, name, shortname, xpr, xpg, xpb, xpa)
        end
    end
end

function core:createTooltip(itemid, iconpath, icon, color, name, shortname, xpr, xpg, xpb, xpa)
    local line, x = tooltip:GetLineCount(), 2
    if core.db.profile.showZero or core:CharHasElement(itemid) then
        line = line + 1
        local item
        if not core.db.profile.tooltipShowShort then
            item  = "|cff"..color..name.."|r"
        else
            item = "|cff"..color..shortname.."|r"
        end
        if core.db.profile.tooltipIcons and not core.db.profile.tooltipIconsOnly then
            item = icon .. " " .. item
        elseif core.db.profile.tooltipIcons and core.db.profile.tooltipIconsOnly then
            item = icon
        else
            item = item
        end
        tooltip:SetCell(tooltip:AddLine(), 1, item.."   ")
        char = self.charName
        tooltip:SetCell(line, 2, "   "..core:CountString(itemid, char))
        if core.db.profile.lineColor then tooltip:SetLineColor(line, xpr, xpg, xpb, xpa) end
        x = 2
        if core.db.profile.showAlts then
            for char, _ in pairs(core.db.realm.chars) do
                if char ~= self.charName then
                    x = x + 1
                    tooltip:SetCell(line, x, "   "..core:CountString(itemid, char))
                end
            end
        end
    end
end